import json
from collections import OrderedDict
from typing import Dict

from abejacli.configuration.config import Config


class ConfigFormatter(object):

    def __init__(self, config: Config):
        super().__init__()
        self.__config = config

    @staticmethod
    def build(config: Config, format: str = None):
        if format == 'json':
            return JSONConfigFormatter(config)
        else:
            return PlainConfigFormatter(config)

    @property
    def config(self) -> Config:
        return self.__config

    def format(self, user: bool = False, token: bool = False, organization: bool = False) -> str:
        config = self.config

        if user:
            return self.format_dict({'abeja-platform-user': config.user})
        elif token:
            return self.format_dict({'personal-access-token': config.token})
        elif organization:
            return self.format_dict({'organization-name': config.organization})
        else:
            return self.format_dict(OrderedDict([
                ('abeja-platform-user', config.user),
                ('personal-access-token', config.token),
                ('organization-name', config.organization),
            ]))

    def format_dict(self, dict: Dict[str, str]) -> str:
        raise NotImplementedError


class PlainConfigFormatter(ConfigFormatter):

    def format_dict(self, dict: Dict[str, str]) -> str:
        items = ['{}:{}'.format(k, v) for k, v in dict.items()]
        return '\n'.join(items)


class JSONConfigFormatter(ConfigFormatter):

    def format_dict(self, dict: Dict[str, str]) -> str:
        return json.dumps(dict, indent=2)
